"""Mutation operators for pytest-gremlins.

This package provides the operator system for defining and managing
mutation operators that inject gremlins into Python code.
"""

from pytest_gremlins.operators.arithmetic import ArithmeticOperator
from pytest_gremlins.operators.boolean import BooleanOperator
from pytest_gremlins.operators.boundary import BoundaryOperator
from pytest_gremlins.operators.comparison import ComparisonOperator
from pytest_gremlins.operators.protocol import GremlinOperator
from pytest_gremlins.operators.registry import OperatorRegistry
from pytest_gremlins.operators.return_value import ReturnOperator

__all__ = [
    'ArithmeticOperator',
    'BooleanOperator',
    'BoundaryOperator',
    'ComparisonOperator',
    'GremlinOperator',
    'OperatorRegistry',
    'ReturnOperator',
]
